/*
 * C BIGNUM library.
 *
 * Syktyvkar State University, 2011 (c) copyleft
 *
 * Applied Mathematics and Programming laboratory.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef BIGNUM_H_
#define BIGNUM_H_

#define BIGNUM_BASE	65536

#define BIGNUM_OK	0
#define BIGNUM_ERROR	1
#define BIGNUM_NOMEM	2

struct bignum {
	int		sign;	/* 0 - positive, 1 - negative */
	int		top;	/* last digit index */
	int		len;	/* number of allocated places for digits */
	unsigned short *val;	/* digits by BIGNUM_BASE */
};

/* Allocates new bignum structure. */
struct bignum *bnum_new();

/* Releases bignum structure and it's resources. */
void bnum_free(struct bignum **bn);

/* Copies bignum from src to dst. */
int bnum_copy(struct bignum *dst, const struct bignum *src);

/* Sets bignum digits using array of values. */
int bnum_set(struct bignum *bn, const unsigned short *val, int count, int neg);

/* Sets bignum digits using array of values with additional checks.  */
int bnum_set_safe(struct bignum *bn, const unsigned short *val, int count, int neg);

/* Resets resources used by bignum structure, to look like a new allocated bignum. */
void bnum_reset(struct bignum *bn);

/* Zeroes bignum value. */
void bnum_clear(struct bignum *bn);

/* Initializes statically allocated bignum structure. */ 
void bnum_init_static(struct bignum *bn);

/* Basic arithmetic operations. */

/* Compares two numbers, returns 1, -1 or 0 if a > b, a < b and a == b. */
int bnum_cmp(const struct bignum *a, const struct bignum *b);

/* Adds two bignum numbers: c = a + b. */
int bnum_add(struct bignum *c, const struct bignum *a, const struct bignum *b);

/* Subtracts two bignum numbers: c = a - b. */
int bnum_sub(struct bignum *c, const struct bignum *a, const struct bignum *b);

/* Multiplies two bignum numbers: c = a * b.  */
int bnum_mul(struct bignum *c, const struct bignum *a, const struct bignum *b);

/* Multiplies two bignum numbers: c = a * b(karatsuba alg).  */
int bnum_kmul(struct bignum *c, struct bignum *a, struct bignum *b);

/* Shift number left to "shift" bits. */
int bnum_shl(struct bignum *bn, unsigned int shift);

/* Shift number right to "shift" bits. */
int bnum_shr(struct bignum *bn, unsigned int shift);

#endif
